@@ -1,6 +1,6 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id: launcher.c 21876 2006-05-28 15:41:24Z jasper $
+/*  $Id: launcher.c 22697 2006-08-09 18:36:00Z jasper $
  *
  *  Copyright © 2005 Jasper Huijsmans <jasper@xfce.org>
  *
@@ -339,7 +339,7 @@ launcher_icon_load_pixbuf (GtkWidget *w, LauncherIcon * icon, int size)
 LauncherEntry *
 launcher_entry_new (void)
 {
-    return g_new0 (LauncherEntry, 1);
+    return panel_slice_new0 (LauncherEntry);
 }
 
 void
@@ -352,7 +352,7 @@ launcher_entry_free (LauncherEntry *e)
     g_free (e->exec);
     g_free (e->real_exec);
 
-    g_free (e);
+    panel_slice_free (LauncherEntry, e);
 }
 
 static void
@@ -578,12 +578,6 @@ launcher_menu_item_activate (GtkWidget *mi, LauncherEntry *entry)
     launcher_entry_exec (gtk_widget_get_screen (mi), entry);
 }
 
-static void
-menu_detached (void)
-{
-    /* do nothing */
-}
-
 static gboolean
 launcher_menu_drag_leave_timeout (LauncherPlugin *launcher)
 {
@@ -614,28 +608,6 @@ launcher_menu_drag_leave (GtkWidget *w, GdkDragContext *drag_context,
                    launcher);
 }
 
-static void
-launcher_create_menu (LauncherPlugin *launcher)
-{
-    launcher->menu = gtk_menu_new ();
-
-    g_signal_connect (launcher->menu, "button-release-event", 
-                      G_CALLBACK (launcher_button_released),
-                      launcher);
-    
-    g_signal_connect (launcher->menu, "deactivate", 
-                      G_CALLBACK (launcher_menu_deactivated), launcher);
-
-    gtk_menu_attach_to_widget (GTK_MENU (launcher->menu), 
-                               launcher->arrowbutton,
-                               (GtkMenuDetachFunc)menu_detached);
-
-    launcher_set_drag_dest (launcher->menu);
-    
-    g_signal_connect (launcher->menu, "drag-leave", 
-                      G_CALLBACK (launcher_menu_drag_leave), launcher);
-}
-
 static void
 launcher_destroy_menu (LauncherPlugin *launcher)
 {
@@ -645,22 +617,34 @@ launcher_destroy_menu (LauncherPlugin *launcher)
 }
 
 static gboolean
-load_menu_icon (GtkImageMenuItem *mi)
+load_menu_icons (LauncherPlugin *launcher)
 {
-    GtkWidget *img;
+    GtkWidget *img, *mi;
     GdkPixbuf *pb;
     LauncherEntry *entry;
+    int i;
+    GList *children, *li;
     
-    if ((entry = g_object_get_data (G_OBJECT (mi), "launcher_entry")) != NULL)
+    if (!launcher->menu)
+        return FALSE;
+
+    children = gtk_container_get_children (GTK_CONTAINER (launcher->menu));
+
+    for (li = children, i = 1; li != 0; li = li->next, ++i)
     {
+        entry = g_ptr_array_index (launcher->entries, i);
+        mi = li->data;
+
         pb = launcher_icon_load_pixbuf (GTK_WIDGET (mi), &entry->icon, 
                                         MENU_ICON_SIZE);
         img = gtk_image_new_from_pixbuf (pb);
         gtk_widget_show (img);
         gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), img);
-        g_object_unref (pb);
+        g_object_unref (G_OBJECT (pb));
     }
 
+    g_list_free (children);
+
     return FALSE;
 }
 
@@ -678,8 +662,8 @@ launcher_recreate_menu (LauncherPlugin *launcher)
         return;
     }
     
-    launcher_create_menu (launcher);
-    
+    launcher->menu = gtk_menu_new ();
+
     for (i = launcher->entries->len - 1; i > 0; --i)
     {
         GtkWidget *mi;
@@ -692,7 +676,6 @@ launcher_recreate_menu (LauncherPlugin *launcher)
 
         /* delayed loading of icons */
         g_object_set_data (G_OBJECT (mi), "launcher_entry", entry);
-        g_idle_add ((GSourceFunc) load_menu_icon, mi);
         
         g_signal_connect (mi, "button-release-event", 
                           G_CALLBACK (launcher_button_released),
@@ -711,6 +694,23 @@ launcher_recreate_menu (LauncherPlugin *launcher)
         g_signal_connect (mi, "drag-leave", 
                           G_CALLBACK (launcher_menu_drag_leave), launcher);
     }
+
+    g_signal_connect (launcher->menu, "button-release-event", 
+                      G_CALLBACK (launcher_button_released),
+                      launcher);
+    
+    g_signal_connect (launcher->menu, "deactivate", 
+                      G_CALLBACK (launcher_menu_deactivated), launcher);
+
+    launcher_set_drag_dest (launcher->menu);
+    
+    g_signal_connect (launcher->menu, "drag-leave", 
+                      G_CALLBACK (launcher_menu_drag_leave), launcher);
+    
+    if (launcher->entries->len > 1)
+    {
+        g_idle_add ((GSourceFunc) load_menu_icons, launcher);
+    }
 }
 
 void
@@ -732,7 +732,7 @@ launcher_update_panel_entry (LauncherPlugin *launcher)
     
     pb = launcher_icon_load_pixbuf (launcher->image, &(entry->icon), size);
     gtk_image_set_from_pixbuf (GTK_IMAGE (launcher->image), pb);
-    g_object_unref (pb);
+    g_object_unref (G_OBJECT (pb));
     
     if (entry->name)
     {
@@ -750,13 +750,6 @@ launcher_update_panel_entry (LauncherPlugin *launcher)
     gtk_tooltips_set_tip (launcher->tips, launcher->iconbutton, tip, NULL);
 }
 
-static gboolean
-update_panel_entry_idle (LauncherPlugin *launcher)
-{
-    launcher_update_panel_entry (launcher);
-    return FALSE;
-}
-
 static void
 launcher_position_menu (GtkMenu * menu, int *x, int *y, gboolean * push_in,
                         GtkWidget *b)
@@ -798,6 +791,8 @@ launcher_position_menu (GtkMenu * menu, int *x, int *y, gboolean * push_in,
             *y += widget->allocation.y - req.height
                 + widget->allocation.height;
             break;
+        default:
+            break;
     }
 
     screen = gtk_widget_get_screen (widget);
@@ -1044,7 +1039,7 @@ launcher_read_rc_file (XfcePanelPlugin *plugin, LauncherPlugin *launcher)
     
     xfce_rc_close (rc);
     
-    g_idle_add ((GSourceFunc)update_panel_entry_idle, launcher);
+    launcher_update_panel_entry (launcher);
     launcher_recreate_menu (launcher);
 }
 
@@ -1201,10 +1196,6 @@ plugin_icon_theme_changed (GtkWidget *w, gpointer ignored,
 
     launcher_update_panel_entry (launcher);
 
-    /* improve interactivity */
-    while (gtk_events_pending ())
-        gtk_main_iteration();
-    
     launcher_recreate_menu (launcher);
 }
 
@@ -1220,12 +1211,12 @@ launcher_new (XfcePanelPlugin *plugin)
     size = xfce_panel_plugin_get_size (plugin);
     screen_position = xfce_panel_plugin_get_screen_position (plugin);
     
-    launcher = g_new0 (LauncherPlugin, 1);
+    launcher = panel_slice_new0 (LauncherPlugin);
     
     launcher->plugin = GTK_WIDGET (plugin);
     
     launcher->tips = gtk_tooltips_new ();
-    g_object_ref (launcher->tips);
+    g_object_ref (G_OBJECT (launcher->tips));
     gtk_object_sink (GTK_OBJECT (launcher->tips));
     
     launcher->entries = g_ptr_array_new ();
@@ -1304,14 +1295,14 @@ launcher_new (XfcePanelPlugin *plugin)
 
     if (launcher->entries->len == 0)
     {
-        LauncherEntry *entry = entry = g_new0 (LauncherEntry, 1);
+        LauncherEntry *entry = entry = panel_slice_new0 (LauncherEntry);
 
         entry->name = g_strdup (_("New Item"));
         entry->comment = g_strdup (_("This item has not yet been configured"));
 
         g_ptr_array_add (launcher->entries, entry);
 
-        g_idle_add ((GSourceFunc)update_panel_entry_idle, launcher);
+        launcher_update_panel_entry (launcher);
     }
     else if (launcher->entries->len > 1)
     {
@@ -1331,7 +1322,7 @@ launcher_free (LauncherPlugin *launcher)
 {
     int i;
 
-    g_object_unref (launcher->tips);
+    g_object_unref (G_OBJECT (launcher->tips));
     
     for (i = 0; i < launcher->entries->len; ++i)
     {
@@ -1346,5 +1337,5 @@ launcher_free (LauncherPlugin *launcher)
         gtk_widget_destroy (launcher->menu);
     
     launcher->plugin = NULL;
-    g_free (launcher);
+    panel_slice_free (LauncherPlugin, launcher);
 }
